#!/usr/bin/env node
const fs = require("fs");
const crypto = require("crypto");
const readline = require("readline");

console.log("\
  ____                 _ _ _____            \n\
 / _  | ___  _   _  __| | |____ | __ _ ___  \n\
| | | |/ _ \\| | | |/ _` | | |_  |/ _` |__ \\ \n\
| |_| | (_) | |_| | (_| | |___| | | | |__) |\n\
 \\____|\\___/|_.__/ \\__,_|_|_____|_| |_|___/ \n\
                                            \n\
  Ready");

var rl = readline.createInterface(process.stdin, process.stdout);

rl.setPrompt('> ');
rl.prompt();

var inputFile = false;
var outputFile = false;
var passphrase = false;

var state = 0;

rl.on('line', function (input) {
    input = input.trim();
    
    if (state == 0) {
        if (input == "exit") {
            state = -1;
        } else if (input == "encrypt") {
            state = 11;
            rl.setPrompt("input file: ");
        } else if (input == "decrypt") {
            state = 21;
            rl.setPrompt("input file: ");
        } else {
            console.log("  encrypt");
            console.log("  decrypt");
            console.log("  exit");
            console.log("");
        }
    } else if (state == 11) {
        inputFile = input;
        rl.setPrompt("output file: ");
        state = 12;
    } else if (state == 21) {
        inputFile = input;
        rl.setPrompt("output file: ");
        state = 22;
    } else if (state == 12) {
        outputFile = input;
        rl.setPrompt("passphrase: ");
        state = 13;
    } else if (state == 22) {
        outputFile = input;
        rl.setPrompt("passphrase: ");
        state = 23;
    } else if (state == 13) {
        passphrase = input;
        state = 14;
    } else if (state == 23) {
        passphrase = input;
        state = 24;
    }
    
    if (state == 14) {
        if (! fs.existsSync(inputFile)) {
            console.log(inputFile + " does not exist");
            state = 0;
            rl.setPrompt('> ');
        } else {
            var key = crypto.createHash('sha512').update(passphrase, 'utf8').digest();
            var input = fs.readFileSync(inputFile);
            
            console.log("First pass...");
            var output = Buffer.from(input);
            for (var i = 0; i < output.length; i++) {
                output[i] += key[i % key.length];
            }
            
            console.log("Second pass...");
            input = Buffer.from(output.toString('hex'));
            var output = Buffer.from(input);
            for (var i = 0; i < output.length; i++) {
                output[i] += key[i % key.length];
            }
            
            fs.writeFileSync(outputFile, output);
            console.log("Written " + output.length + " bytes");
            
            state = 0;
            rl.setPrompt('> ');
        }
    } else if (state == 24) {
        if (! fs.existsSync(inputFile)) {
            console.log(inputFile + " does not exist");
            state = 0;
            rl.setPrompt('> ');
        } else {
            var key = crypto.createHash('sha512').update(passphrase, 'utf8').digest();
            var input = fs.readFileSync(inputFile);
            
            console.log("First pass...");
            var output = Buffer.from(input);
            for (var i = 0; i < output.length; i++) {
                output[i] -= key[i % key.length];
            }
            
            console.log("Second pass...");
            input = Buffer.from(output.toString(), 'hex');
            var output = Buffer.from(input);
            for (var i = 0; i < output.length; i++) {
                output[i] -= key[i % key.length];
            }
            
            fs.writeFileSync(outputFile, output);
            console.log("Written " + output.length + " bytes");
            
            state = 0;
            rl.setPrompt('> ');
        }
    }
    
    if (state == -1) {
        process.exit();
    }

    rl.prompt();
});
